/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcbuildxvcddiscstructurejob.h"

#include "../kdcconfig.h"
#include "../kdcnamespace.h"
#include "../kdcfilefactory.h"

#include <QList>
#include <QTreeWidgetItem>
#include <QStringList>
#include <QByteArray>

#include <KLocale>

KDCBuildXVCDDiscStructureJob::KDCBuildXVCDDiscStructureJob( KDCXVCDDisc *disc, QObject *parent )
	: KDCJob( parent )
{
	p_disc = disc;

	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	p_process->setWorkingDirectory( KDCFileFactory::discDirectory( disc ) );

	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );

	QStringList arguments;
	arguments << "-n" << QString::number( KDCConfig::niceLevel() );
	arguments << "vcdxbuild" << "-p";
	arguments << "-c" << KDCFileFactory::discOutputDirectory( disc ) + "/disc.cue";
	arguments << "-b" << KDCFileFactory::discOutputDirectory( disc ) + "/disc.bin";
	arguments << KDCFileFactory::discConfigFile( disc );

	p_process->setProgram( "nice", arguments );

	int frameCount = 0;
	QList<QTreeWidgetItem*> list = disc->takeMovies();
	QList<QTreeWidgetItem*>::const_iterator it;
	KDCMovie *movie;
	KDCMovieGroup *group;
	for ( it = list.constBegin(); it != list.constEnd(); ++it )
	{
		if ( ( *it )->type() == KDC::Movie )
		{
			movie = static_cast<KDCMovie*>( *it );
			//frameCount += movie->videoStream()->metaData()->frameCount();
			frameCount += qRound( movie->videoStream()->metaData()->length()/1000.0*movie->fps() );
		}
		else
		{
			group = static_cast<KDCMovieGroup*>( *it );
			frameCount += qRound( group->length()/1000.0*group->movie( 0 )->fps() );
		}
	}

	setTotalSize( frameCount );
	qDebug( "KDCBuildXVCDDiscStructureJob::frameCount = %d", frameCount );
	qDebug( arguments.join(" ").toUtf8() );
	p_totalSum = 0;
	p_prevSize = 0;
	p_const = (double)frameCount/( ( disc->topLevelItemCount() + 1 )*100.0 );
}

KDCBuildXVCDDiscStructureJob::~KDCBuildXVCDDiscStructureJob()
{
	if ( p_process )
	{
		p_process->disconnect();
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
}

void KDCBuildXVCDDiscStructureJob::start()
{
	emit jobMainProgram( "vcdxbuild" );
	emit jobTotalSize( totalSize() );
	emit percent( 0 );
	emit infoMessage( i18n("Start building disc structure") );
	p_process->start();
	p_timerID = startTimer( KDCConfig::updateData() );
}

void KDCBuildXVCDDiscStructureJob::on_readyReadStandardOutput()
{
/*
	QByteArray data = p_process->readAllStandardOutput();
	int idx = data.indexOf( '(', 17 );
	int err = data.indexOf( "ERROR" );
	if ( idx != -1 && err == -1 )
	{
		idx += 1;
		QByteArray tmp;
		char byte = data.at( idx );
		while ( byte != '%' )
		{
			tmp.append( byte );
			++idx;
			byte = data.at( idx );
		}
		tmp.trimmed();
		int size = tmp.toInt();
		if ( p_prevSize <= size )
			p_totalSum += size - p_prevSize;
		else
			p_totalSum += size;
		emit speed( ( p_totalSum - p_prevSize )*p_const );
		p_prevSize = size;
		emit progress( qRound( p_totalSum*p_const ) );
	}
	else
		if ( err != -1 )
			p_process->close();
*/
}

void KDCBuildXVCDDiscStructureJob::on_finished( int code, QProcess::ExitStatus status )
{
	killTimer( p_timerID );
	if ( status == QProcess::CrashExit )
	{
		setError( i18n("There was an error during process. Error code is %1").arg( code ) );
		KDCFileFactory::removeFile( KDCFileFactory::discOutputDirectory( p_disc ) + "/disc.cue" );
		KDCFileFactory::removeFile( KDCFileFactory::discOutputDirectory( p_disc ) + "/disc.bin" );
	}
	else
	{
		emit infoMessage( i18n("Builded disc structure sucesfully finished") );
		emit percent( 100 );
		if ( p_disc->settings()->deleteTempFilesEnabled() )
		{
			QList<QTreeWidgetItem*> movies = p_disc->takeMovies();
			QList<QTreeWidgetItem*>::const_iterator it;
			for ( it = movies.constBegin(); it != movies.constEnd(); ++it )
			{
				if ( ( *it )->type() == KDC::Movie )
					KDCFileFactory::removeFile( KDCFileFactory::movieName( static_cast<KDCMovie*>( *it ) ) );
				else
					KDCFileFactory::removeFile( KDCFileFactory::movieName( static_cast<KDCMovieGroup*>( *it ) ) );
			}
		}
	}
	KDCFileFactory::removeFile( KDCFileFactory::discConfigFile( p_disc ) );

	emit finished( this );
}

void KDCBuildXVCDDiscStructureJob::timerEvent( QTimerEvent *event )
{
	if ( event->timerId() == p_timerID )
	{
		QByteArray data = p_process->readAllStandardOutput();
		int idx = data.indexOf( '(', 17 );
		int err = data.indexOf( "ERROR" );
		if ( idx != -1 && err == -1 )
		{
			idx += 1;
			QByteArray tmp;
			char byte = data.at( idx );
			while ( byte != '%' )
			{
				tmp.append( byte );
				++idx;
				byte = data.at( idx );
			}
			tmp.trimmed();
			int size = tmp.toInt();
			if ( p_prevSize <= size )
				p_totalSum += size - p_prevSize;
			else
				p_totalSum += size;
			emit speed( ( p_totalSum - p_prevSize )*p_const );
			p_prevSize = size;
			emit progress( qRound( p_totalSum*p_const ) );
		}
		else
			if ( err != -1 )
				p_process->close();
	}
}
