/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmovie.h"

#include "kdcsubtitlestream.h"
#include "lib/kdclib.h"
#include "kdcdisc.h"
#include "kdcfilefactory.h"

#include <QFile>

#include <KLocale>
#include <KIcon>
#include <kio/job.h>

KDCMovie::KDCMovie( KDCMovie *movie )
	: QObject( 0 ), QTreeWidgetItem( KDC::Movie )
{
	p_updates = movie->updatesMode();
	p_fileName = movie->fileName();
	p_videoStream = new KDCVideoStream( movie->videoStream() );
	p_format = movie->videoFormat();
	p_fps = movie->fps();
	if ( movie->audioStreamGroup() )
		p_audioGroup = new KDCAudioStreamGroup( movie->audioStreamGroup() );
	else
		p_audioGroup = NULL;
	p_subGroup = new KDCSubtitleStreamGroup( this );
	p_chapterGroup = new KDCChapterGroup( movie->chapterGroup() );
}

void KDCMovie::copySettings( const KDCMovie *movie )
{
	p_updates = KDC::Disabled;
	p_fps = movie->fps();
	p_format = movie->videoFormat();
	p_videoStream->copySettings( movie->videoStream() );
	if ( p_audioGroup )
		p_audioGroup->copySettings( movie->audioStreamGroup() );
	p_updates = KDC::Enabled;
	refresh();
	p_updates = movie->updatesMode();
}

KDCMovie::KDCMovie( const QString& fileName,
										KDCVideoStreamMetaData *videoMetaData,
										const QList<KDCAudioStreamMetaData*>& audioMetaData )
	: QObject( 0 ), QTreeWidgetItem( KDC::Movie )
{
	p_updates = KDC::Disabled;
	p_format = KDC::PAL;
	p_fps = 25.0;
	p_fileName = fileName;
	p_subGroup = new KDCSubtitleStreamGroup( this );
	p_group = NULL;
	p_disc = NULL;
	p_chapterGroup = NULL;

	p_videoStream = new KDCVideoStream( this, videoMetaData );
	connect( p_videoStream, SIGNAL( settingsChanged() ), this, SLOT( refresh() ) );
	if ( audioMetaData.size() > 0 )
	{
		p_audioGroup = new KDCAudioStreamGroup;
		KDCAudioStream *audio;
		for( int i=0; i< audioMetaData.size(); ++i )
		{
			audio = new KDCAudioStream( this, audioMetaData.at( i ) );
			p_audioGroup->addStream( audio );
			connect( audio, SIGNAL( settingsChanged() ), this, SLOT( refresh() ) );
		}
	}
	else
		p_audioGroup = NULL;

	p_outputName = KDCFileFactory::baseFileName( p_fileName );
	setOutputName( p_outputName );
	setToolTip( 0, i18n("Movie Clip") );
	setIcon( 0 , KIcon("applications-multimedia") );
	QTreeWidgetItem *item = new QTreeWidgetItem( KDC::MovieInfo );
	addChild( item );
	setFlags( Qt::ItemIsEnabled | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled );
	item->setFlags( Qt::ItemIsEnabled | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled );
	item->setToolTip( 0, i18n("Movie Clip") );
}

KDCMovie::~KDCMovie()
{
	delete p_videoStream;
	delete p_subGroup;
	if ( p_audioGroup )
		delete p_audioGroup;
	if ( p_chapterGroup )
		delete p_chapterGroup;
}

QString KDCMovie::fileName() const
{
	return p_fileName;
}

KDCDisc* KDCMovie::disc() const
{
	return p_disc;
}

int KDCMovie::id() const
{
	return p_id;
}

KDCMovieGroup* KDCMovie::movieGroup() const
{
	return p_group;
}

KDCChapterGroup* KDCMovie::chapterGroup() const
{
	return p_chapterGroup;
}

void KDCMovie::checkSubtitles()
{
	QStringList subs;
	QString str = KDCFileFactory::fileNameWithoutExtension( fileName() );
	if ( QFile::exists( str ) )
		subs << str;
	if ( QFile::exists( str  + ".txt" ) )
		subs << str + ".txt";
	if ( QFile::exists( str  + ".sub" ) )
		subs << str + ".sub";
	if ( QFile::exists( str  + ".str" ) )
		subs << str + ".srt";
 qDebug( subs.join(" ").toUtf8() );
	if ( subs.size() > 0 )
	{
		for( int i=0; i<subs.size(); ++i )
			p_subGroup->addStream( new KDCSubtitleStream( subs.at( i ), p_videoStream->metaData()->fps() ) );
	}
}

void KDCMovie::setFps( double fps )
{
	if ( p_fps != fps )
	{
		qDebug( "KDCMovie::setFps( %f )", fps );
		p_fps = fps;
		p_videoStream->refresh();
	}
}

double KDCMovie::fps() const
{
	return p_fps;
}

KDCVideoStream* KDCMovie::videoStream() const
{
	return p_videoStream;
}

KDCAudioStreamGroup* KDCMovie::audioStreamGroup() const
{
	return p_audioGroup;
}

KDCSubtitleStreamGroup* KDCMovie::subtitleStreamGroup() const
{
	return p_subGroup;
}

void KDCMovie::setVideoFormat( KDC::VideoFormat format )
{
	if ( p_format != format )
	{
		qDebug( "KDCMovie::setVideoFormat( %d )", format );
		p_format = format;
		if ( format == KDC::PAL )
			p_fps = 25.0;
		else
			if ( format == KDC::NTSC && p_fps == 25.0 )
				p_fps = 29.970;
		if ( disc()->discType() == KDC::DVD && format != KDC::PAL )
		{
			if ( p_audioGroup )
			{
				KDCAudioStream *stream;
				for ( int i=0; i<p_audioGroup->count(); ++i )
				{
					stream = p_audioGroup->stream( i );
					if ( stream->codec() == KDC::MP2 )
					{
						p_updates = KDC::Disabled;
						stream->setCodec( KDC::AC3 );
						stream->refresh();
					}
				}
			}
		}
		p_videoStream->setResolution( p_videoStream->width(), KDCLib::getCorrectHeight(	p_videoStream->width(),
																																										p_videoStream->height(),
																																										p_format ) );
		p_updates = KDC::Enabled;
		refresh();
	}
}

KDC::VideoFormat KDCMovie::videoFormat() const
{
	return p_format;
}

void KDCMovie::setUpdatesMode( KDC::Mode mode )
{
	if ( p_updates != mode )
		p_updates = mode;
}

KDC::Mode KDCMovie::updatesMode() const
{
	return p_updates;
}

void KDCMovie::refresh()
{
	if ( p_updates == KDC::Enabled )
	{
		qDebug( "KDCMovie::refresh()::begin" );
		QString info;
		info.append( "i\t" + KDCFileFactory::movieFrame( this ) + '\t' );
		p_size = p_videoStream->size();
		info.append( p_videoStream->info() );

		if ( p_audioGroup )
		{
			KDCAudioStream *audio;
			for( int i=0; i<p_audioGroup->count(); ++i )
			{
				audio = p_audioGroup->stream( i );
				if ( audio->enabledMode() == KDC::Enabled )
				{
					p_size +=audio->size();
					info.append( '\t' );
					info.append( audio->info() );
				}
			}
		}
		child( 0 )->setText( 0, info + "\n\n\n\n\n\n\n\n\n" );

		setOutputName( p_outputName );
		qDebug( "KDCMovie::refresh()::emit settingsChanged()" );
		emit settingsChanged();
		qDebug( "KDCMovie::refresh()::end" );
	}
}

quint64 KDCMovie::size() const
{
	return p_size;
}

bool KDCMovie::operator!=( const KDCMovie& movie )
{
	if ( this && &movie )
	{
		if ( p_format != movie.videoFormat() )
			return TRUE;
		if ( p_fps != movie.fps() )
			return TRUE;
		if ( *p_videoStream != *movie.videoStream() )
			return TRUE;
		if ( *p_audioGroup != *movie.audioStreamGroup() )
			return TRUE;
		if ( p_subGroup->count() != movie.subtitleStreamGroup()->count() )
			return TRUE;
	}
	else
		if ( ( this != NULL && &movie == NULL ) || ( this == NULL && &movie != NULL ) )
			return TRUE;

	return FALSE;
}

QString KDCMovie::outputName() const
{
	return p_outputName;
}

void KDCMovie::setOutputName( const QString& name )
{
	p_outputName = name;
	setText( 0, QString( "m\t" + p_outputName + "\t" + p_fileName + '\t' + KIO::convertSizeFromKiB( p_size ) + "\n" ) );
}
