/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcqualityeditor.h"

#include "../../kdcnamespace.h"
#include "../../kdcdisc.h"
#include "../../lib/kdclib.h"

#include <QStringList>
#include <QHeaderView>
#include <QTreeWidgetItem>

#include <KLocale>

KDCQualityEditor::KDCQualityEditor( QWidget *parent )
 : QTreeWidget( parent )
{
	p_movie = NULL;

	setRootIsDecorated( FALSE );
	setAlternatingRowColors( TRUE );
	setColumnCount( 2 );

	QStringList headers;
	headers << i18n("Property") << i18n("Value");
	setHeaderLabels( headers );
	header()->setDefaultAlignment( Qt::AlignHCenter | Qt::AlignVCenter );

	QTreeWidgetItem *item = new QTreeWidgetItem;
	item->setText( 0, i18n( "Create Higher Quality DVD" ) );
	addTopLevelItem( item );
	item = new QTreeWidgetItem;
	item->setText( 0, i18n( "Use the closest GOP" ) );
	addTopLevelItem( item );
	item = new QTreeWidgetItem;
	item->setText( 0, i18n( "Use Trellis Searched Quantizations" ) );
	addTopLevelItem( item );
	item = new QTreeWidgetItem;
	item->setText( 0, i18n( "Macro Block decision algorithm" ) );
	addTopLevelItem( item );
	item = new QTreeWidgetItem;
	item->setText( 0, i18n( "Deinterlacing Filter ( LCD TV )" ) );
	addTopLevelItem( item );

	p_macroMap.insert( KDC::MBCMP, i18n("Use MBCMP (faster)") );
	p_macroMap.insert( KDC::MDB1, i18n("Select the MB mode which needs the fewest bits") );
	p_macroMap.insert( KDC::MDB2, i18n("Select the MB mode which has the best rate distortion") );

	p_deintMap.insert( KDC::None, i18n("Don't deinterlate") );
	p_deintMap.insert( KDC::Linear, i18n("Linear blend deinterlacing filter") );
	p_deintMap.insert( KDC::Median, i18n("Median deinterlacing filter") );
	p_deintMap.insert( KDC::FFmpeg, i18n("FFmpeg deinterlacing filter ( default for RMVB )") );

	p_macroDelegate = new KDCVideoQualityDelegate( KDCVideoQualityDelegate::MacroBlock );
	connect( p_macroDelegate, SIGNAL( macroBlockModeChanged( KDC::MacroBlockMode ) ), this, SLOT( on_macroBlockModeChanged( KDC::MacroBlockMode ) ) );
	connect( p_macroDelegate, SIGNAL( closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ), this, SLOT( on_closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ) );
	p_deintDelegate = new KDCVideoQualityDelegate( KDCVideoQualityDelegate::Deinterlace );
	connect( p_deintDelegate, SIGNAL( deinterlaceModeChanged( KDC::DeinterlaceMode ) ), this, SLOT( on_deinterlaceModeChanged( KDC::DeinterlaceMode ) ) );
	connect( p_deintDelegate, SIGNAL( closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ), this, SLOT( on_closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ) );

	connect( this, SIGNAL( itemDoubleClicked( QTreeWidgetItem *, int ) ), this, SLOT( openEditor( QTreeWidgetItem *, int ) ) );
	connect( this, SIGNAL( itemClicked( QTreeWidgetItem *, int ) ), this, SLOT( on_itemClicked( QTreeWidgetItem *, int ) ) );
}

KDCQualityEditor::~KDCQualityEditor()
{
	delete p_macroDelegate;
	delete p_deintDelegate;
}

void KDCQualityEditor::resizeEvent( QResizeEvent *event )
{
	setColumnWidth( 0, 350 );
	setColumnWidth( 1, qMax( 0, event->size().width() - 350 ) );
}

void KDCQualityEditor::setCurrentMovie( KDCMovieGroup *movie )
{
	setCurrentMovie( movie->KDCMovieGroup::movie( 0 ) );
}

void KDCQualityEditor::setCurrentMovie( KDCMovie *movie )
{
	clean();
	if ( movie )
	{
		qDebug( "KDCQualityEditor::setCurrentMovie( 1 )" );
		p_movie = movie;
		connect( p_movie->videoStream(), SIGNAL( settingsChanged() ), this, SLOT( readMovieSettings() ) );
		readMovieSettings();
	}
}

void KDCQualityEditor::clean()
{
	qDebug( "KDCQualityEditor::clean()" );
	if ( p_movie )
		disconnect( p_movie->videoStream(), SIGNAL( settingsChanged() ), this, SLOT( readMovieSettings() ) );
	p_movie = NULL;
	uint i;
	for ( i=0; i<3; ++i )
	{
		topLevelItem( i )->setCheckState( 1, Qt::Unchecked );
		topLevelItem( i )->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsUserCheckable );
	}
	for ( i=3; i<5; ++i )
	{
		topLevelItem( i )->setText( 1, "" );
		topLevelItem( i )->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled );
	}
}

void KDCQualityEditor::readMovieSettings()
{
	if ( p_movie )
	{
		qDebug( "KDCQualityEditor::readMovieSettings()::begin" );
		if ( p_movie->videoStream()->mode() == KDC::Copy )
		{
			setEnabled( FALSE );
			return;
		}
		else
		{
			setEnabled( TRUE );

			if ( p_movie->videoStream()->hqDVDMode() == KDC::Enabled )
			{
				topLevelItem( 0 )->setCheckState( 1, Qt::Checked );
				topLevelItem( 2 )->setFlags( Qt::ItemIsSelectable );
				topLevelItem( 3 )->setFlags( Qt::ItemIsSelectable );
			}
			else
			{
				topLevelItem( 0 )->setCheckState( 1, Qt::Unchecked );
				topLevelItem( 2 )->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsUserCheckable );
				topLevelItem( 3 )->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled );
			}
			if ( p_movie->videoStream()->trellisMode() == KDC::Enabled )
				topLevelItem( 2 )->setCheckState( 1, Qt::Checked );
			else
				topLevelItem( 2 )->setCheckState( 1, Qt::Unchecked );
			if ( p_movie->videoStream()->closestGOPMode() == KDC::Enabled )
				topLevelItem( 1 )->setCheckState( 1, Qt::Checked );
			else
				topLevelItem( 1 )->setCheckState( 1, Qt::Unchecked );

			if ( ( p_movie->disc()->discType() == KDC::DVD ) && ( p_movie->videoStream()->codec() == KDC::mpeg2 ) )
				topLevelItem( 0 )->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsUserCheckable );
			else
				topLevelItem( 0 )->setFlags( Qt::ItemIsSelectable );

			topLevelItem( 3 )->setText( 1, p_macroMap.value( p_movie->videoStream()->macroBlockMode() ) );
			topLevelItem( 4 )->setText( 1, p_deintMap.value( p_movie->videoStream()->deinterlaceMode() ) );

			p_deintDelegate->setCurrentData( p_movie );
			p_macroDelegate->setCurrentData( p_movie );
		}
		qDebug( "KDCQualityEditor::readMovieSettings::end" );
	}
}

void KDCQualityEditor::on_macroBlockModeChanged( KDC::MacroBlockMode mode )
{
	if ( p_movie )
		p_movie->videoStream()->setMacroBlockMode( mode );
}

void KDCQualityEditor::on_deinterlaceModeChanged( KDC::DeinterlaceMode mode )
{
	if ( p_movie )
		p_movie->videoStream()->setDeinterlaceMode( mode );
}

void KDCQualityEditor::on_closeEditor( QWidget *w, QAbstractItemDelegate::EndEditHint h )
{
	qDebug( "KDCQualityEditor::closeEditor()" );
	currentItem()->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled );
}

void KDCQualityEditor::openEditor( QTreeWidgetItem *item, int column )
{
	if ( p_movie )
	{
		int row = indexOfTopLevelItem( item );
		if ( column == 1 && row > 2 )
		{
			switch ( row )
			{
				case 3:
				{
					setItemDelegateForColumn( 1, p_macroDelegate );
					break;
				}
				case 4:
				{
					setItemDelegateForColumn( 1, p_deintDelegate );
					break;
				}
			}
			item->setFlags( item->flags() | Qt::ItemIsEditable );
			editItem( item, column );
		}
	}
}

void KDCQualityEditor::on_itemClicked( QTreeWidgetItem *item, int column )
{
	if ( p_movie )
	{
		int row = indexOfTopLevelItem( item );
		if ( row < 3 && column == 1 )
		{
			if ( item->checkState( 1 ) == Qt::Checked )
			{
				if ( row == 0 )
					p_movie->videoStream()->setHQDVDMode( KDC::Enabled );
				else
					if ( row == 1 )
						p_movie->videoStream()->setClosestGOPMode( KDC::Enabled );
				else
					p_movie->videoStream()->setTrellisMode( KDC::Enabled );
			}
			else
			{
				if ( row == 0 )
					p_movie->videoStream()->setHQDVDMode( KDC::Disabled );
				else
					if ( row == 1 )
						p_movie->videoStream()->setClosestGOPMode( KDC::Disabled );
				else
					p_movie->videoStream()->setTrellisMode( KDC::Disabled );
			}
		}
	}
}
